"""
SPDX-License-Identifier: MIT

  Copyright (c) 2024, SCANOSS

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
"""

import os
import shutil
import tempfile
import unittest

from scanoss.file_filters import FileFilters
from scanoss.scanoss_settings import ScanossSettings


class TestFileFilters(unittest.TestCase):
    def setUp(self):
        self.file_filters = FileFilters(debug=True, hidden_files_folders=True, operation_type='scanning')
        self.test_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.test_dir)

    def create_files(self, files):
        for file in files:
            file_path = os.path.join(self.test_dir, file)
            os.makedirs(os.path.dirname(file_path), exist_ok=True)
            with open(file_path, 'w') as f:
                f.write('test')

    def test_default_extensions(self):
        files = [
            'file1.js',
            'file2.go',
            'file3.py',
            'file4.css',  # Should be skipped by default
            'file5.doc',  # Should be skipped by default
            'dir1/file6.py',
            'dir1/file7.go',
            'dir2/file8.js',
            'dir2/file9.csv',  # Should be skipped by default
        ]
        self.create_files(files)

        expected_files = [
            'file1.js',
            'file2.go',
            'file3.py',
            'dir1/file6.py',
            'dir1/file7.go',
            'dir2/file8.js',
        ]

        filtered_files = self.file_filters.get_filtered_files_from_folder(self.test_dir)
        self.assertEqual(sorted(filtered_files), sorted(expected_files))

    def test_default_folders(self):
        files = [
            '__pycache__/file1.pyc',
            'venv/file2.py',
            'dir1/nbdist/test.py',
            'dir1/eggs/test1.py',
            'dir1/file3.py',
            'dir1/file4.go',
            'dir2/wheels/test.js',
            'dir2/file5.js',
            'package.egg-info/file6.py',
        ]
        self.create_files(files)

        expected_files = [
            'dir1/file3.py',
            'dir1/file4.go',
            'dir2/file5.js',
        ]

        filtered_files = self.file_filters.get_filtered_files_from_folder(self.test_dir)
        self.assertEqual(sorted(filtered_files), sorted(expected_files))

    def test_default_skipped_files(self):
        files = [
            'gradlew',
            'gradlew.bat',
            'mvnw',
            'license.txt',
            'makefile',
            'normal_file.py',
            'dir1/gradle-wrapper.jar',
            'dir1/normal_file.js',
        ]
        self.create_files(files)

        expected_files = [
            'normal_file.py',
            'dir1/normal_file.js',
        ]

        filtered_files = self.file_filters.get_filtered_files_from_folder(self.test_dir)
        self.assertEqual(sorted(filtered_files), sorted(expected_files))

    def test_size_limits(self):
        settings = ScanossSettings()
        settings.data = {
            'settings': {
                'skip': {
                    'sizes': {
                        'scanning': [{'patterns': ['*.py'], 'min': 150, 'max': 450}],
                        'fingerprinting': [{'patterns': ['*'], 'min': 150, 'max': 450}],
                    }
                }
            }
        }
        files = [
            'file1.js',  # 100 bytes
            'file2.py',  # 200 bytes - within limits
            'file3.py',  # 500 bytes - exceeds max
            'file4.py',  # 100 bytes - below min
        ]

        for file in files:
            file_path = os.path.join(self.test_dir, file)
            with open(file_path, 'w') as f:
                if 'file1' in file:
                    f.write('a' * 100)
                elif 'file2' in file:
                    f.write('a' * 200)
                elif 'file3' in file:
                    f.write('a' * 500)
                else:
                    f.write('a' * 100)

        file_filters = FileFilters(
            debug=True, scanoss_settings=settings, hidden_files_folders=True, operation_type='scanning'
        )

        # For scanning, only *.py files have size limits
        filtered_files = file_filters.get_filtered_files_from_folder(self.test_dir)
        self.assertEqual(sorted(filtered_files), ['file1.js', 'file2.py'])

        file_filters = FileFilters(
            debug=True, scanoss_settings=settings, hidden_files_folders=True, operation_type='fingerprinting'
        )

        # For fingerprinting, all files have size limits
        filtered_files = file_filters.get_filtered_files_from_folder(self.test_dir)
        self.assertEqual(sorted(filtered_files), ['file2.py'])

    def test_all_extensions_flag(self):
        file_filters = FileFilters(
            debug=True, all_extensions=True, hidden_files_folders=True, operation_type='scanning'
        )
        files = [
            'file1.js',
            'file2.css',  # Would normally be skipped
            'file3.doc',  # Would normally be skipped
            'dir1/file4.csv',  # Would normally be skipped
        ]
        self.create_files(files)

        filtered_files = file_filters.get_filtered_files_from_folder(self.test_dir)
        self.assertEqual(sorted(filtered_files), sorted(files))

    def test_all_folders_flag(self):
        file_filters = FileFilters(debug=True, all_folders=True, hidden_files_folders=True, operation_type='scanning')

        files = [
            '__pycache__/file1.py',  # Would normally be skipped
            'venv/file2.py',  # Would normally be skipped
            'dir1/nbdist/file3.py',  # Would normally be skipped
            'dir1/file4.py',
        ]
        self.create_files(files)

        filtered_files = file_filters.get_filtered_files_from_folder(self.test_dir)
        self.assertEqual(sorted(filtered_files), sorted(files))

    def test_get_filtered_files_from_files(self):
        files = [
            os.path.join(self.test_dir, 'file1.js'),
            os.path.join(self.test_dir, 'file2.css'),  # Should be skipped
            os.path.join(self.test_dir, 'dir1/file3.py'),
            os.path.join(self.test_dir, 'dir1/__pycache__/file4.py'), # Should be skipped
        ]
        self.create_files(files)

        filtered_files = self.file_filters.get_filtered_files_from_files(files, self.test_dir)

        expected_files = [
            'file1.js',
            'dir1/file3.py',
        ]
        self.assertEqual(sorted(filtered_files), sorted(expected_files))

    def test_hidden_files_and_folders_enabled(self):
        files = [
            '.hidden_file.py',
            '.hidden_dir/visible_file.py',
            '.hidden_dir/.nested_hidden_file.js',
            'visible_dir/.hidden_file.go',
            '.git/config',
            '.hidden_dir/nested_dir/.hidden_nested_file.py',
        ]
        self.create_files(files)

        expected_files = [
            '.hidden_file.py',
            '.hidden_dir/visible_file.py',
            '.hidden_dir/.nested_hidden_file.js',
            'visible_dir/.hidden_file.go',
            '.hidden_dir/nested_dir/.hidden_nested_file.py',
        ]

        filtered_files = self.file_filters.get_filtered_files_from_folder(self.test_dir)
        self.assertEqual(sorted(filtered_files), sorted(expected_files))

    def test_hidden_files_and_folders_disabled(self):
        file_filters = FileFilters(debug=True, hidden_files_folders=False, operation_type='scanning')
        files = [
            '.hidden_file.py',
            '.hidden_dir/visible_file.py',
            '.hidden_dir/.nested_hidden_file.js',
            'visible_dir/.hidden_file.go',
            'visible_file.py',
            '.git/config',
        ]
        self.create_files(files)

        expected_files = ['visible_file.py']

        filtered_files = file_filters.get_filtered_files_from_folder(self.test_dir)
        self.assertEqual(sorted(filtered_files), sorted(expected_files))

    def test_all_extensions_mode(self):
        file_filters = FileFilters(debug=True, all_extensions=True, hidden_files_folders=True)
        files = [
            'file1.css',
            'file2.doc',
            'file3.csv',
            '.hidden_file.dat',
            'dir1/file4.bmp',
            'dir1/.hidden/file5.class',
            'file6.py',
        ]
        self.create_files(files)

        expected_files = [
            'file1.css',
            'file2.doc',
            'file3.csv',
            '.hidden_file.dat',
            'dir1/file4.bmp',
            'dir1/.hidden/file5.class',
            'file6.py',
        ]

        filtered_files = file_filters.get_filtered_files_from_folder(self.test_dir)
        self.assertEqual(sorted(filtered_files), sorted(expected_files))

    def test_all_folders_mode(self):
        file_filters = FileFilters(debug=True, all_folders=True, hidden_files_folders=True, operation_type='scanning')
        files = [
            '__pycache__/cache.py',
            'venv/lib.py',
            'eggs/module.py',
            'wheels/util.py',
            'normal_dir/file.py',
            '.git/config.py',
        ]
        self.create_files(files)

        expected_files = [
            '__pycache__/cache.py',
            'venv/lib.py',
            'eggs/module.py',
            'wheels/util.py',
            'normal_dir/file.py',
            '.git/config.py',
        ]

        filtered_files = file_filters.get_filtered_files_from_folder(self.test_dir)
        self.assertEqual(sorted(filtered_files), sorted(expected_files))

    def test_combined_all_modes(self):
        file_filters = FileFilters(
            debug=True, all_extensions=True, all_folders=True, hidden_files_folders=True, operation_type='scanning'
        )
        files = [
            '.hidden_dir/file1.css',
            '__pycache__/cache.dat',
            'venv/.hidden_file.class',
            'normal_dir/file.py',
            '.config/settings.bmp',
        ]
        self.create_files(files)

        expected_files = [
            '.hidden_dir/file1.css',
            '__pycache__/cache.dat',
            'venv/.hidden_file.class',
            'normal_dir/file.py',
            '.config/settings.bmp',
        ]

        filtered_files = file_filters.get_filtered_files_from_folder(self.test_dir)
        self.assertEqual(sorted(filtered_files), sorted(expected_files))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: scanoss/api/cryptography/v2/scanoss-cryptography.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'scanoss/api/cryptography/v2/scanoss-cryptography.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from scanoss.api.common.v2 import scanoss_common_pb2 as scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n6scanoss/api/cryptography/v2/scanoss-cryptography.proto\x12\x1bscanoss.api.cryptography.v2\x1a*scanoss/api/common/v2/scanoss-common.proto\x1a\x1cgoogle/api/annotations.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\"0\n\tAlgorithm\x12\x11\n\talgorithm\x18\x01 \x01(\t\x12\x10\n\x08strength\x18\x02 \x01(\t\"\xf3\x01\n\x11\x41lgorithmResponse\x12\x43\n\x05purls\x18\x01 \x03(\x0b\x32\x34.scanoss.api.cryptography.v2.AlgorithmResponse.Purls\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1a\x62\n\x05Purls\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12:\n\nalgorithms\x18\x03 \x03(\x0b\x32&.scanoss.api.cryptography.v2.Algorithm\"\x82\x02\n\x19\x41lgorithmsInRangeResponse\x12J\n\x05purls\x18\x01 \x03(\x0b\x32;.scanoss.api.cryptography.v2.AlgorithmsInRangeResponse.Purl\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1a\x62\n\x04Purl\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x10\n\x08versions\x18\x02 \x03(\t\x12:\n\nalgorithms\x18\x03 \x03(\x0b\x32&.scanoss.api.cryptography.v2.Algorithm\"\xe1\x01\n\x17VersionsInRangeResponse\x12H\n\x05purls\x18\x01 \x03(\x0b\x32\x39.scanoss.api.cryptography.v2.VersionsInRangeResponse.Purl\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1a\x45\n\x04Purl\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x15\n\rversions_with\x18\x02 \x03(\t\x12\x18\n\x10versions_without\x18\x03 \x03(\t\"b\n\x04Hint\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x04 \x01(\t\x12\x0b\n\x03url\x18\x05 \x01(\t\x12\x0c\n\x04purl\x18\x06 \x01(\t\"\xe1\x01\n\rHintsResponse\x12?\n\x05purls\x18\x01 \x03(\x0b\x32\x30.scanoss.api.cryptography.v2.HintsResponse.Purls\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1aX\n\x05Purls\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x30\n\x05hints\x18\x03 \x03(\x0b\x32!.scanoss.api.cryptography.v2.Hint\"\xee\x01\n\x14HintsInRangeResponse\x12\x45\n\x05purls\x18\x01 \x03(\x0b\x32\x36.scanoss.api.cryptography.v2.HintsInRangeResponse.Purl\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1aX\n\x04Purl\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x10\n\x08versions\x18\x02 \x03(\t\x12\x30\n\x05hints\x18\x03 \x03(\x0b\x32!.scanoss.api.cryptography.v2.Hint2\x88\x07\n\x0c\x43ryptography\x12u\n\x04\x45\x63ho\x12\".scanoss.api.common.v2.EchoRequest\x1a#.scanoss.api.common.v2.EchoResponse\"$\x82\xd3\xe4\x93\x02\x1e\"\x19/api/v2/cryptography/echo:\x01*\x12\x8f\x01\n\rGetAlgorithms\x12\".scanoss.api.common.v2.PurlRequest\x1a..scanoss.api.cryptography.v2.AlgorithmResponse\"*\x82\xd3\xe4\x93\x02$\"\x1f/api/v2/cryptography/algorithms:\x01*\x12\xa5\x01\n\x14GetAlgorithmsInRange\x12\".scanoss.api.common.v2.PurlRequest\x1a\x36.scanoss.api.cryptography.v2.AlgorithmsInRangeResponse\"1\x82\xd3\xe4\x93\x02+\"&/api/v2/cryptography/algorithmsInRange:\x01*\x12\x9f\x01\n\x12GetVersionsInRange\x12\".scanoss.api.common.v2.PurlRequest\x1a\x34.scanoss.api.cryptography.v2.VersionsInRangeResponse\"/\x82\xd3\xe4\x93\x02)\"$/api/v2/cryptography/versionsInRange:\x01*\x12\x96\x01\n\x0fGetHintsInRange\x12\".scanoss.api.common.v2.PurlRequest\x1a\x31.scanoss.api.cryptography.v2.HintsInRangeResponse\",\x82\xd3\xe4\x93\x02&\"!/api/v2/cryptography/hintsInRange:\x01*\x12\x8b\x01\n\x12GetEncryptionHints\x12\".scanoss.api.common.v2.PurlRequest\x1a*.scanoss.api.cryptography.v2.HintsResponse\"%\x82\xd3\xe4\x93\x02\x1f\"\x1a/api/v2/cryptography/hints:\x01*B\x9e\x02Z9github.com/scanoss/papi/api/cryptographyv2;cryptographyv2\x92\x41\xdf\x01\x12y\n\x1cSCANOSS Cryptography Service\"T\n\x14scanoss-cryptography\x12\'https://github.com/scanoss/crpytography\x1a\x13support@scanoss.com2\x03\x32.0*\x01\x01\x32\x10\x61pplication/json:\x10\x61pplication/jsonR;\n\x03\x34\x30\x34\x12\x34\n*Returned when the resource does not exist.\x12\x06\n\x04\x9a\x02\x01\x07\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scanoss.api.cryptography.v2.scanoss_cryptography_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z9github.com/scanoss/papi/api/cryptographyv2;cryptographyv2\222A\337\001\022y\n\034SCANOSS Cryptography Service\"T\n\024scanoss-cryptography\022\'https://github.com/scanoss/crpytography\032\023support@scanoss.com2\0032.0*\001\0012\020application/json:\020application/jsonR;\n\003404\0224\n*Returned when the resource does not exist.\022\006\n\004\232\002\001\007'
  _globals['_CRYPTOGRAPHY'].methods_by_name['Echo']._loaded_options = None
  _globals['_CRYPTOGRAPHY'].methods_by_name['Echo']._serialized_options = b'\202\323\344\223\002\036\"\031/api/v2/cryptography/echo:\001*'
  _globals['_CRYPTOGRAPHY'].methods_by_name['GetAlgorithms']._loaded_options = None
  _globals['_CRYPTOGRAPHY'].methods_by_name['GetAlgorithms']._serialized_options = b'\202\323\344\223\002$\"\037/api/v2/cryptography/algorithms:\001*'
  _globals['_CRYPTOGRAPHY'].methods_by_name['GetAlgorithmsInRange']._loaded_options = None
  _globals['_CRYPTOGRAPHY'].methods_by_name['GetAlgorithmsInRange']._serialized_options = b'\202\323\344\223\002+\"&/api/v2/cryptography/algorithmsInRange:\001*'
  _globals['_CRYPTOGRAPHY'].methods_by_name['GetVersionsInRange']._loaded_options = None
  _globals['_CRYPTOGRAPHY'].methods_by_name['GetVersionsInRange']._serialized_options = b'\202\323\344\223\002)\"$/api/v2/cryptography/versionsInRange:\001*'
  _globals['_CRYPTOGRAPHY'].methods_by_name['GetHintsInRange']._loaded_options = None
  _globals['_CRYPTOGRAPHY'].methods_by_name['GetHintsInRange']._serialized_options = b'\202\323\344\223\002&\"!/api/v2/cryptography/hintsInRange:\001*'
  _globals['_CRYPTOGRAPHY'].methods_by_name['GetEncryptionHints']._loaded_options = None
  _globals['_CRYPTOGRAPHY'].methods_by_name['GetEncryptionHints']._serialized_options = b'\202\323\344\223\002\037\"\032/api/v2/cryptography/hints:\001*'
  _globals['_ALGORITHM']._serialized_start=209
  _globals['_ALGORITHM']._serialized_end=257
  _globals['_ALGORITHMRESPONSE']._serialized_start=260
  _globals['_ALGORITHMRESPONSE']._serialized_end=503
  _globals['_ALGORITHMRESPONSE_PURLS']._serialized_start=405
  _globals['_ALGORITHMRESPONSE_PURLS']._serialized_end=503
  _globals['_ALGORITHMSINRANGERESPONSE']._serialized_start=506
  _globals['_ALGORITHMSINRANGERESPONSE']._serialized_end=764
  _globals['_ALGORITHMSINRANGERESPONSE_PURL']._serialized_start=666
  _globals['_ALGORITHMSINRANGERESPONSE_PURL']._serialized_end=764
  _globals['_VERSIONSINRANGERESPONSE']._serialized_start=767
  _globals['_VERSIONSINRANGERESPONSE']._serialized_end=992
  _globals['_VERSIONSINRANGERESPONSE_PURL']._serialized_start=923
  _globals['_VERSIONSINRANGERESPONSE_PURL']._serialized_end=992
  _globals['_HINT']._serialized_start=994
  _globals['_HINT']._serialized_end=1092
  _globals['_HINTSRESPONSE']._serialized_start=1095
  _globals['_HINTSRESPONSE']._serialized_end=1320
  _globals['_HINTSRESPONSE_PURLS']._serialized_start=1232
  _globals['_HINTSRESPONSE_PURLS']._serialized_end=1320
  _globals['_HINTSINRANGERESPONSE']._serialized_start=1323
  _globals['_HINTSINRANGERESPONSE']._serialized_end=1561
  _globals['_HINTSINRANGERESPONSE_PURL']._serialized_start=1473
  _globals['_HINTSINRANGERESPONSE_PURL']._serialized_end=1561
  _globals['_CRYPTOGRAPHY']._serialized_start=1564
  _globals['_CRYPTOGRAPHY']._serialized_end=2468
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from scanoss.api.common.v2 import scanoss_common_pb2 as scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2
from scanoss.api.licenses.v2 import scanoss_licenses_pb2 as scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2

GRPC_GENERATED_VERSION = '1.73.1'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in scanoss/api/licenses/v2/scanoss_licenses_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class LicenseStub(object):
    """
    License Service Definition
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Echo = channel.unary_unary(
                '/scanoss.api.licenses.v2.License/Echo',
                request_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoResponse.FromString,
                _registered_method=True)
        self.GetComponentLicenses = channel.unary_unary(
                '/scanoss.api.licenses.v2.License/GetComponentLicenses',
                request_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.ComponentLicenseResponse.FromString,
                _registered_method=True)
        self.GetComponentsLicenses = channel.unary_unary(
                '/scanoss.api.licenses.v2.License/GetComponentsLicenses',
                request_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentsRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.ComponentsLicenseResponse.FromString,
                _registered_method=True)
        self.GetDetails = channel.unary_unary(
                '/scanoss.api.licenses.v2.License/GetDetails',
                request_serializer=scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.LicenseRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.LicenseDetailsResponse.FromString,
                _registered_method=True)
        self.GetObligations = channel.unary_unary(
                '/scanoss.api.licenses.v2.License/GetObligations',
                request_serializer=scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.LicenseRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.ObligationsResponse.FromString,
                _registered_method=True)


class LicenseServicer(object):
    """
    License Service Definition
    """

    def Echo(self, request, context):
        """
        Returns the same message that was sent, used for health checks and connectivity testing
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetComponentLicenses(self, request, context):
        """
        Get license information for a single software component.

        Examines source code, license files, and package metadata to determine which licenses apply to the component.
        Returns license data in both individual SPDX license and SPDX expressions when determinable.

        See: https://github.com/scanoss/papi/blob/main/protobuf/scanoss/api/licenses/v2/README.md?tab=readme-ov-file#getcomponentlicenses
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetComponentsLicenses(self, request, context):
        """
        Get license information for multiple software components in a single request.

        Examines source code, license files, and package metadata to determine which licenses apply to each component.
        Returns license data in both individual SPDX license and SPDX expressions when determinable.

        See https://github.com/scanoss/papi/blob/main/protobuf/scanoss/api/licenses/v2/README.md?tab=readme-ov-file#getcomponentslicenses
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDetails(self, request, context):
        """
        Get detailed metadata for a specific license by SPDX identifier.

        Provides comprehensive license information including SPDX registry data,
        OSADL compliance metadata, license type classification, and official references.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetObligations(self, request, context):
        """
        Get compliance obligations and usage requirements for a specific license.

        Returns structured OSADL compliance data including use cases, obligations,
        compatibility information, and patent hints for the specified license.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_LicenseServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Echo': grpc.unary_unary_rpc_method_handler(
                    servicer.Echo,
                    request_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoResponse.SerializeToString,
            ),
            'GetComponentLicenses': grpc.unary_unary_rpc_method_handler(
                    servicer.GetComponentLicenses,
                    request_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.ComponentLicenseResponse.SerializeToString,
            ),
            'GetComponentsLicenses': grpc.unary_unary_rpc_method_handler(
                    servicer.GetComponentsLicenses,
                    request_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentsRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.ComponentsLicenseResponse.SerializeToString,
            ),
            'GetDetails': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDetails,
                    request_deserializer=scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.LicenseRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.LicenseDetailsResponse.SerializeToString,
            ),
            'GetObligations': grpc.unary_unary_rpc_method_handler(
                    servicer.GetObligations,
                    request_deserializer=scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.LicenseRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.ObligationsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'scanoss.api.licenses.v2.License', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('scanoss.api.licenses.v2.License', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class License(object):
    """
    License Service Definition
    """

    @staticmethod
    def Echo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.licenses.v2.License/Echo',
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoRequest.SerializeToString,
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetComponentLicenses(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.licenses.v2.License/GetComponentLicenses',
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentRequest.SerializeToString,
            scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.ComponentLicenseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetComponentsLicenses(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.licenses.v2.License/GetComponentsLicenses',
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentsRequest.SerializeToString,
            scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.ComponentsLicenseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetDetails(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.licenses.v2.License/GetDetails',
            scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.LicenseRequest.SerializeToString,
            scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.LicenseDetailsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetObligations(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.licenses.v2.License/GetObligations',
            scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.LicenseRequest.SerializeToString,
            scanoss_dot_api_dot_licenses_dot_v2_dot_scanoss__licenses__pb2.ObligationsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: scanoss/api/geoprovenance/v2/scanoss-geoprovenance.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'scanoss/api/geoprovenance/v2/scanoss-geoprovenance.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from scanoss.api.common.v2 import scanoss_common_pb2 as scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n8scanoss/api/geoprovenance/v2/scanoss-geoprovenance.proto\x12\x1cscanoss.api.geoprovenance.v2\x1a*scanoss/api/common/v2/scanoss-common.proto\x1a\x1cgoogle/api/annotations.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\"\xd1\x03\n\x13\x43ontributorResponse\x12\x46\n\x05purls\x18\x01 \x03(\x0b\x32\x37.scanoss.api.geoprovenance.v2.ContributorResponse.Purls\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1a\x32\n\x10\x44\x65\x63laredLocation\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x10\n\x08location\x18\x02 \x01(\t\x1a\x31\n\x0f\x43uratedLocation\x12\x0f\n\x07\x63ountry\x18\x01 \x01(\t\x12\r\n\x05\x63ount\x18\x02 \x01(\x05\x1a\xd3\x01\n\x05Purls\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12^\n\x12\x64\x65\x63lared_locations\x18\x02 \x03(\x0b\x32\x42.scanoss.api.geoprovenance.v2.ContributorResponse.DeclaredLocation\x12\\\n\x11\x63urated_locations\x18\x03 \x03(\x0b\x32\x41.scanoss.api.geoprovenance.v2.ContributorResponse.CuratedLocation\"\x99\x02\n\x0eOriginResponse\x12\x41\n\x05purls\x18\x01 \x03(\x0b\x32\x32.scanoss.api.geoprovenance.v2.OriginResponse.Purls\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1a,\n\x08Location\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\npercentage\x18\x02 \x01(\x02\x1a_\n\x05Purls\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12H\n\tlocations\x18\x02 \x03(\x0b\x32\x35.scanoss.api.geoprovenance.v2.OriginResponse.Location2\xb9\x03\n\rGeoProvenance\x12v\n\x04\x45\x63ho\x12\".scanoss.api.common.v2.EchoRequest\x1a#.scanoss.api.common.v2.EchoResponse\"%\x82\xd3\xe4\x93\x02\x1f\"\x1a/api/v2/geoprovenance/echo:\x01*\x12\x9d\x01\n\x18GetComponentContributors\x12\".scanoss.api.common.v2.PurlRequest\x1a\x31.scanoss.api.geoprovenance.v2.ContributorResponse\"*\x82\xd3\xe4\x93\x02$\"\x1f/api/v2/geoprovenance/countries:\x01*\x12\x8f\x01\n\x12GetComponentOrigin\x12\".scanoss.api.common.v2.PurlRequest\x1a,.scanoss.api.geoprovenance.v2.OriginResponse\"\'\x82\xd3\xe4\x93\x02!\"\x1c/api/v2/geoprovenance/origin:\x01*B\xa4\x02Z;github.com/scanoss/papi/api/geoprovenancev2;geoprovenancev2\x92\x41\xe3\x01\x12}\n\x1eSCANOSS GEO Provenance Service\"V\n\x15scanoss-geoprovenance\x12(https://github.com/scanoss/geoprovenance\x1a\x13support@scanoss.com2\x03\x32.0*\x01\x01\x32\x10\x61pplication/json:\x10\x61pplication/jsonR;\n\x03\x34\x30\x34\x12\x34\n*Returned when the resource does not exist.\x12\x06\n\x04\x9a\x02\x01\x07\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scanoss.api.geoprovenance.v2.scanoss_geoprovenance_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z;github.com/scanoss/papi/api/geoprovenancev2;geoprovenancev2\222A\343\001\022}\n\036SCANOSS GEO Provenance Service\"V\n\025scanoss-geoprovenance\022(https://github.com/scanoss/geoprovenance\032\023support@scanoss.com2\0032.0*\001\0012\020application/json:\020application/jsonR;\n\003404\0224\n*Returned when the resource does not exist.\022\006\n\004\232\002\001\007'
  _globals['_GEOPROVENANCE'].methods_by_name['Echo']._loaded_options = None
  _globals['_GEOPROVENANCE'].methods_by_name['Echo']._serialized_options = b'\202\323\344\223\002\037\"\032/api/v2/geoprovenance/echo:\001*'
  _globals['_GEOPROVENANCE'].methods_by_name['GetComponentContributors']._loaded_options = None
  _globals['_GEOPROVENANCE'].methods_by_name['GetComponentContributors']._serialized_options = b'\202\323\344\223\002$\"\037/api/v2/geoprovenance/countries:\001*'
  _globals['_GEOPROVENANCE'].methods_by_name['GetComponentOrigin']._loaded_options = None
  _globals['_GEOPROVENANCE'].methods_by_name['GetComponentOrigin']._serialized_options = b'\202\323\344\223\002!\"\034/api/v2/geoprovenance/origin:\001*'
  _globals['_CONTRIBUTORRESPONSE']._serialized_start=213
  _globals['_CONTRIBUTORRESPONSE']._serialized_end=678
  _globals['_CONTRIBUTORRESPONSE_DECLAREDLOCATION']._serialized_start=363
  _globals['_CONTRIBUTORRESPONSE_DECLAREDLOCATION']._serialized_end=413
  _globals['_CONTRIBUTORRESPONSE_CURATEDLOCATION']._serialized_start=415
  _globals['_CONTRIBUTORRESPONSE_CURATEDLOCATION']._serialized_end=464
  _globals['_CONTRIBUTORRESPONSE_PURLS']._serialized_start=467
  _globals['_CONTRIBUTORRESPONSE_PURLS']._serialized_end=678
  _globals['_ORIGINRESPONSE']._serialized_start=681
  _globals['_ORIGINRESPONSE']._serialized_end=962
  _globals['_ORIGINRESPONSE_LOCATION']._serialized_start=821
  _globals['_ORIGINRESPONSE_LOCATION']._serialized_end=865
  _globals['_ORIGINRESPONSE_PURLS']._serialized_start=867
  _globals['_ORIGINRESPONSE_PURLS']._serialized_end=962
  _globals['_GEOPROVENANCE']._serialized_start=965
  _globals['_GEOPROVENANCE']._serialized_end=1406
# @@protoc_insertion_point(module_scope)



"""
SPDX-License-Identifier: MIT

  Copyright (c) 2021, SCANOSS

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
"""

import datetime
import getpass
import hashlib
import json
import os.path
import re
import sys

import importlib_resources
from packageurl import PackageURL

from . import __version__


class SpdxLite:
    """
    SPDX Lite management class
    Handle all interaction with SPDX Lite formatting
    """

    def __init__(self, debug: bool = False, output_file: str = None):
        """
        Initialise the SpdxLite class
        """
        self.output_file = output_file
        self.debug = debug
        self._spdx_licenses = {}  # Used to lookup for valid SPDX license identifiers
        self._spdx_lic_names = {}  # Used to look for SPDX license identifiers by name

    @staticmethod
    def print_stderr(*args, **kwargs):
        """
        Print the given message to STDERR
        """
        print(*args, file=sys.stderr, **kwargs)

    def print_debug(self, *args, **kwargs):
        """
        Print debug message if enabled
        """
        if self.debug:
            self.print_stderr(*args, **kwargs)

    def parse(self, data: json):
        """
        Parse the given input (raw/plain) JSON string and return a summary
        :param data: json - JSON object
        :return: summary dictionary
        """
        if data is None:
            self.print_stderr('ERROR: No JSON data provided to parse.')
            return None
        if len(data) == 0:
            self.print_debug('Warning: Empty scan results provided. Returning empty summary.')
            return {}

        self.print_debug('Processing raw results into summary format...')
        return self._process_files(data)

    def _process_files(self, data: json) -> dict:
        """
            Process raw results and build a component summary.

            Args:
                data: JSON data containing raw results

            Returns:
                dict: The built summary dictionary
        """
        summary = {}
        for file_path in data:
            file_details = data.get(file_path)
            # summary is passed by reference and modified inside the function
            self._process_entries(file_path, file_details, summary)
        return summary

    def _process_entries(self, file_path: str, file_details: list, summary: dict):
        """
        Process entries for a single file.

        Args:
            file_path: Path to the file being processed
            file_details: Results of the file
            summary: Reference to summary dictionary that will be modified in place
        """
        for entry in file_details:
            id_details = entry.get('id')
            if not id_details or id_details == 'none':
                continue

            if id_details == 'dependency':
                self._process_dependency_entry(file_path, entry, summary)
            else:
                self._process_file_entry(file_path, entry, summary)

    def _process_dependency_entry(self, file_path: str, entry: dict, summary: dict):
        """
        Process a dependency type entry.

        Args:
            file_path: Path to the file being processed
            entry: The dependency entry to process
            summary: Reference to summary dictionary that will be modified in place
        """
        dependencies = entry.get('dependencies')
        if not dependencies:
            self.print_stderr(f'Warning: No Dependencies found for {file_path}')
            return

        for dep in dependencies:
            purl = dep.get('purl')
            if not self._is_valid_purl(file_path, dep, purl, summary):
                continue
            # Modifying the summary dictionary directly as it's passed by reference
            summary[purl] = self._create_dependency_summary(dep)

    def _process_file_entry(self, file_path: str, entry: dict, summary: dict):
        """
        Process file entry.

        Args:
            file_path: Path to the file being processed
            entry: Process file match entry
            summary: Reference to summary dictionary that will be modified in place
        """
        purls = entry.get('purl')
        if not purls:
            self.print_stderr(f'Purl block missing for {file_path}')
            return

        purl = purls[0] if purls else None
        if not self._is_valid_purl(file_path, entry, purl, summary):
            return

        summary[purl] = self._create_file_summary(entry)

    def _is_valid_purl(self, file_path: str, entry: dict, purl: str, summary: dict) -> bool:
        """
        Check if purl is valid and not already processed.

        Args:
            file_path: Path to the file being processed
            entry: The entry containing the PURL
            purl: The PURL to validate
            summary: Reference to summary dictionary to check for existing entries

        Returns:
            bool: True if purl is valid and not already processed
        """
        if not purl:
            self.print_stderr(f'Warning: No PURL found for {file_path}: {entry}')
            return False

        if summary.get(purl):
            self.print_debug(f'Component {purl} already stored: {summary.get(purl)}')
            return False

        return True

    def _create_dependency_summary(self, dep: dict) -> dict:
        """
        Create summary for dependency entry.

        This method extracts relevant fields from a dependency entry and creates a
        standardized summary dictionary. It handles fields like component, version,
        and URL, with special processing for licenses.

        Args:
            dep (dict): The dependency entry containing component information

        Returns:
            dict: A new summary dictionary containing the extracted and processed fields
        """
        summary = {}
        for field in ['component', 'version', 'url']:
            summary[field] = dep.get(field, '')
        summary['licenses'] = self._process_licenses(dep.get('licenses'))
        return summary

    def _create_file_summary(self, entry: dict) -> dict:
        """
        Create summary for file entry.

        This method extracts set of fields from file entry and creates a standardized summary dictionary.

        Args:
            entry (dict): The file entry containing the metadata to summarize

        Returns:
            dict: A new summary dictionary containing all extracted and processed fields
        """
        summary = {}
        fields = ['id', 'vendor', 'component', 'version', 'latest',
                  'url', 'url_hash', 'download_url']
        for field in fields:
            summary[field] = entry.get(field)
        summary['licenses'] = self._process_licenses(entry.get('licenses'))
        return summary

    def _process_licenses(self, licenses: list) -> list:
        """
            Process license information and remove duplicates.

            This method filters license information to include only licenses from trusted sources
            ('component_declared' or 'license_file') and removes any duplicate license names.
            The result is a simplified list of license dictionaries containing only the 'id' field.

            Args:
                licenses (list): A list of license dictionaries, each containing at least 'name'
                                 and 'source' fields. Can be None or empty.

            Returns:
                list: A filtered and deduplicated list of license dictionaries, where each
                      dictionary contains only an 'id' field matching the original license name.
                      Returns an empty list if input is None or empty.
            """
        if not licenses:
            return []

        processed_licenses = []
        seen_names = set()

        for license_info in licenses:
            name = license_info.get('name')
            source = license_info.get('source')
            if source not in ("component_declared", "license_file", "file_header"):
                continue
            if name and name not in seen_names:
                processed_licenses.append({'id': name})
                seen_names.add(name)

        return processed_licenses

    def produce_from_file(self, json_file: str, output_file: str = None) -> bool:
        """
        Parse plain/raw input JSON file and produce SPDX Lite output
        :param json_file:
        :param output_file:
        :return: True if successful, False otherwise
        """
        if not json_file:
            self.print_stderr('ERROR: No JSON file provided to parse.')
            return False
        if not os.path.isfile(json_file):
            self.print_stderr(f'ERROR: JSON file does not exist or is not a file: {json_file}')
            return False
        with open(json_file, 'r') as f:
            success = self.produce_from_str(f.read(), output_file)
        return success

    def produce_from_json(self, data: json, output_file: str = None) -> bool:
        """
        Produce the SPDX Lite output from the input data
        :param data: JSON object
        :param output_file: Output file (optional)
        :return: True if successful, False otherwise
        """
        raw_data = self.parse(data)
        if raw_data is None:
            self.print_stderr('ERROR: No SPDX data returned for the JSON string provided.')
            return False
        if len(raw_data) == 0:
            self.print_debug('Warning: Empty scan results - generating minimal SPDX Lite document with no packages.')

        self.load_license_data()
        spdx_document = self._create_base_document(raw_data)
        self._process_packages(raw_data, spdx_document)
        return self._write_output(spdx_document, output_file)

    def _create_base_document(self, raw_data: dict) -> dict:
        """
            Create the base SPDX document structure.

            This method initializes a new SPDX document with standard fields required by
            the SPDX 2.2 specification. It generates a unique document namespace using
            a hash of the raw data and current timestamp.

            Args:
                raw_data (dict): The raw component data used to create a unique identifier
                                for the document namespace

            Returns:
                dict: A dictionary containing the base SPDX document structure with the
                      following fields:
                      - spdxVersion: The SPDX specification version
                      - dataLicense: The license for the SPDX document itself
                      - SPDXID: The document's unique identifier
                      - name: The name of the SBOM
                      - creationInfo: Information about when and how the document was created
                      - documentNamespace: A unique URI for this document
                      - documentDescribes: List of packages described (initially empty)
                      - hasExtractedLicensingInfos: List of licenses (initially empty)
                      - packages: List of package information (initially empty)
        """
        now = datetime.datetime.utcnow()
        md5hex = hashlib.md5(f'{raw_data}-{now}'.encode('utf-8')).hexdigest()

        return {
            'spdxVersion': 'SPDX-2.2',
            'dataLicense': 'CC0-1.0',
            'SPDXID': 'SPDXRef-DOCUMENT',
            'name': 'SCANOSS-SBOM',
            'creationInfo': self._create_creation_info(now),
            'documentNamespace': f'https://spdx.org/spdxdocs/scanoss-py-{__version__}-{md5hex}',
            'documentDescribes': [],
            'hasExtractedLicensingInfos': [],
            'packages': [],
        }

    def _create_creation_info(self, timestamp: datetime.datetime) -> dict:
        """
            Create the creation info section of an SPDX document.

            This method generates the creation information required by the SPDX specification,
            including timestamps, creator information, and document type.

            Args:
                timestamp (datetime.datetime): The UTC timestamp representing when the
                                              document was created

            Returns:
                dict: A dictionary containing creation information with the following fields:
                      - created: ISO 8601 formatted timestamp
                      - creators: List of entities involved in creating the document
                        (tool, person, and organization)
                      - comment: Additional information about the SBOM type
        """
        return {
            'created': timestamp.strftime('%Y-%m-%dT%H:%M:%SZ'),
            'creators': [
                f'Tool: SCANOSS-PY: {__version__}',
                f'Person: {getpass.getuser()}',
                'Organization: SCANOSS'
            ],
            'comment': 'SBOM Build information - SBOM Type: Build',
        }

    def _process_packages(self, raw_data: dict, spdx_document: dict):
        """
            Process packages and add them to the SPDX document.

            This method iterates through the raw component data, creates package information
            for each component, and adds them to the SPDX document. It also collects
            license references to be processed separately.

            Args:
                raw_data (dict): Dictionary of package data indexed by PURL
                                (Package URL identifiers)
                spdx_document (dict): Reference to the SPDX document being built,
                                     which will be modified in place

            Note:
                This method modifies the spdx_document dictionary in place by:
                1. Adding package information to the 'packages' list
                2. Adding package SPDXIDs to the 'documentDescribes' list
                3. Indirectly populating 'hasExtractedLicensingInfos' via _process_license_refs()
        """
        lic_refs = set()

        for purl, comp in raw_data.items():
            package_info = self._create_package_info(purl, comp, lic_refs)
            spdx_document['packages'].append(package_info)
            spdx_document['documentDescribes'].append(package_info['SPDXID'])

        self._process_license_refs(lic_refs, spdx_document)

    def _create_package_info(self, purl: str, comp: dict, lic_refs: set) -> dict:
        """
            Create package information for SPDX document.

            This method generates a complete package information entry following the SPDX
            specification format. It creates a unique identifier for the package based on
            its PURL and version, processes license information, and formats all required
            fields for the SPDX document.

            Args:
                purl (str): Package URL identifier for the component
                comp (dict): Component information dictionary containing metadata like
                            component name, version, URLs, and license information
                lic_refs (set): Reference to a set that will be populated with license
                               references found in this package. This set is modified in place.

            Returns:
                dict: A dictionary containing all required SPDX package fields including:
                      - name: Component name
                      - SPDXID: Unique identifier for this package within the document
                      - versionInfo: Component version
                      - downloadLocation: URL where the package can be downloaded
                      - homepage: Component homepage URL
                      - licenseDeclared: Formatted license expression
                      - licenseConcluded: NOASSERTION as automated conclusion isn't possible
                      - filesAnalyzed: False as files are not individually analyzed
                      - copyrightText: NOASSERTION as copyright text isn't available
                      - supplier: Organization name from vendor information
                      - externalRefs: Package URL reference for package manager integration
                      - checksums: MD5 hash of the package if available
        """
        lic_text = self._process_package_licenses(comp.get('licenses', []), lic_refs)
        comp_ver = comp.get('version')
        purl_ver = f'{purl}@{comp_ver}'
        purl_hash = hashlib.md5(purl_ver.encode('utf-8')).hexdigest()

        return {
            'name': comp.get('component'),
            'SPDXID': f'SPDXRef-{purl_hash}',
            'versionInfo': comp_ver,
            'downloadLocation': comp.get('download_url') or comp.get('url'),
            'homepage': comp.get('url', ''),
            'licenseDeclared': lic_text,
            'licenseConcluded': 'NOASSERTION',
            'filesAnalyzed': False,
            'copyrightText': 'NOASSERTION',
            'supplier': f'Organization: {comp.get("vendor", "NOASSERTION")}',
            'externalRefs': [
                {
                    'referenceCategory': 'PACKAGE-MANAGER',
                    'referenceLocator': PackageURL.from_string(purl_ver).to_string(),
                    'referenceType': 'purl'
                }
            ],
            'checksums': [
                {
                    'algorithm': 'MD5',
                    'checksumValue': comp.get('url_hash') or '0' * 32
                }
            ],
        }

    def _process_package_licenses(self, licenses: list, lic_refs: set) -> str:
        """
           Process licenses and return license text formatted for SPDX.

           This method processes a list of license objects, extracts valid license IDs,
           converts them to SPDX format, and combines them into a properly formatted
           license expression.

           Args:
               licenses (list): List of license dictionaries, each containing at least
                               an 'id' field
               lic_refs (set): Reference to a set that will collect license references.
                              This set is modified in place.

           Returns:
               str: A formatted license expression string following SPDX syntax.
                    Returns 'NOASSERTION' if no valid licenses are found.
        """
        if not licenses:
            return 'NOASSERTION'

        lic_set = set()
        for lic in licenses:
            lc_id = lic.get('id')
            self._process_license_id(lc_id, lic_refs, lic_set)

        return self._format_license_text(lic_set)

    def _process_license_id(self, lc_id: str, lic_refs: set, lic_set: set):
        """
         Process individual license ID and add to appropriate sets.

         This method attempts to convert a license ID to its SPDX equivalent.
         If not found in the SPDX license list, it's formatted as a LicenseRef
         and added to the license references set.

         Args:
             lc_id (str): The license ID to process
             lic_refs (set): Reference to a set that collects license references
                            for later processing. Modified in place.
             lic_set (set): Reference to a set collecting all license IDs for
         """
        spdx_id = self.get_spdx_license_id(lc_id)
        if not spdx_id:
            if not lc_id.startswith('LicenseRef'):
                lc_id = f'LicenseRef-{lc_id}'
            lic_refs.add(lc_id)
        lic_set.add(spdx_id if spdx_id else lc_id)

    def _format_license_text(self, lic_set: set) -> str:
        """
            Format the license text with proper SPDX syntax.

            This method combines multiple license IDs with the 'AND' operator
            according to SPDX specification rules. If multiple licenses are present,
            the expression is enclosed in parentheses.

            Args:
                lic_set (set): Set of license IDs to format

            Returns:
                str: A properly formatted SPDX license expression.
                     Returns 'NOASSERTION' if the set is empty.
        """
        if not lic_set:
            return 'NOASSERTION'

        lic_text = ' AND '.join(lic_set)
        if len(lic_set) > 1:
            lic_text = f'({lic_text})'
        return lic_text

    def _process_license_refs(self, lic_refs: set, spdx_document: dict):
        """
            Process and add license references to the SPDX document.

            This method processes each license reference in the provided set
            and adds corresponding license information to the SPDX document's
            extracted licensing information section.

            Args:
                lic_refs (set): Set of license references to process
                spdx_document (dict): Reference to the SPDX document being built,
                                     which will be modified in place

            Note:
                This method modifies the spdx_document dictionary in place by adding
                entries to the 'hasExtractedLicensingInfos' list.
        """
        for lic_ref in lic_refs:
            license_info = self._parse_license_ref(lic_ref)
            spdx_document['hasExtractedLicensingInfos'].append(license_info)

    def _parse_license_ref(self, lic_ref: str) -> dict:
        """
            Parse license reference and create info dictionary for SPDX document.

            This method extracts information from a license reference identifier
            and formats it into the structure required by the SPDX specification
            for extracted licensing information.

            Args:
                lic_ref (str): License reference identifier to parse

            Returns:
                dict: Dictionary containing required SPDX fields for extracted license info:
                      - licenseId: The unique identifier for this license
                      - name: A readable name for the license
                      - extractedText: A placeholder for the actual license text
                      - comment: Information about how the license was detected
        """
        source, name = self._extract_license_info(lic_ref)
        source_text = f' by {source}.' if source else '.'

        return {
            'licenseId': lic_ref,
            'name': name.replace('-', ' '),
            'extractedText': 'Detected license, please review component source code.',
            'comment': f'Detected license{source_text}',
        }

    def _extract_license_info(self, lic_ref: str):
        """
            Extract source and name from license reference.

            This method parses a license reference string to extract the source
            (e.g., scancode, scanoss) and the actual license name using regular
            expressions.

            Args:
                lic_ref (str): License reference identifier to parse

            Returns:
                tuple: A tuple containing (source, name) where:
                       - source (str): The tool or system that identified the license
                       - name (str): The actual license name
        """
        match = re.search(r'^LicenseRef-(scancode-|scanoss-|)(\S+)$', lic_ref, re.IGNORECASE)
        if match:
            source = match.group(1).replace('-', '')
            name = match.group(2)
        else:
            source = ''
            name = lic_ref
        return source, name

    def _write_output(self, data: dict, output_file: str = None) -> bool:
        """Write the SPDX document to output."""
        try:
            file = self._get_output_file(output_file)
            print(json.dumps(data, indent=2), file=file)
            if output_file:
                file.close()
            return True
        except Exception as e:
            self.print_stderr(f'Error writing output: {str(e)}')
            return False

    def _get_output_file(self, output_file: str = None):
        """Get the appropriate output file handle."""
        if not output_file and self.output_file:
            output_file = self.output_file
        return open(output_file, 'w') if output_file else sys.stdout

    def produce_from_str(self, json_str: str, output_file: str = None) -> bool:
        """
        Produce SPDX Lite output from input JSON string
        :param json_str: input JSON string
        :param output_file: Output file (optional)
        :return: True if successful, False otherwise
        """
        if not json_str:
            self.print_stderr('ERROR: No JSON string provided to parse.')
            return False
        try:
            data = json.loads(json_str)
        except Exception as e:
            self.print_stderr(f'ERROR: Problem parsing input JSON: {e}')
            return False
        return self.produce_from_json(data, output_file)

    def load_license_data(self) -> None:
        """
        Load the embedded SPDX valid license JSON files
        Parse its contents to provide a lookup for valid name
        """
        # SPDX license files details from: https://spdx.org/licenses/
        # Specifically the JSON files come from GitHub: https://github.com/spdx/license-list-data/tree/master/json
        self._spdx_licenses = {}
        self._spdx_lic_names = {}
        self.print_debug('Loading SPDX License details...')
        self.load_license_data_file('data/spdx-licenses.json')
        self.load_license_data_file('data/spdx-exceptions.json', 'licenseExceptionId')

    def load_license_data_file(self, filename: str, lic_field: str = 'licenseId') -> bool:
        """
        Load the embedded SPDX valid license JSON file
        Parse its contents to provide a lookup for valid name
        :param filename: license data file to load
        :param lic_field: license id field name (default: licenseId)
        :return: True if successful, False otherwise
        """
        try:
            f_name = importlib_resources.files(__name__) / filename
            with importlib_resources.as_file(f_name) as f:
                with open(f, 'r', encoding='utf-8') as file:
                    data = json.load(file)
        except Exception as e:
            self.print_stderr(f'ERROR: Problem parsing SPDX license input JSON: {e}')
            return False
        else:
            licenses = data.get('licenses')
            if licenses:
                for lic in licenses:
                    lic_name = re.sub('\\s+', '', lic.get('name')).lower()
                    lic_id = lic.get(lic_field)
                    if lic_id:
                        lic_id_lc = lic_id.lower()
                        self._spdx_licenses[lic_id_lc] = lic_id
                        lic_id_short = (lic_id_lc.split('-'))[0]  # extract the name minus the version (i.e. SSPL-1.0)
                        if lic_id_lc != lic_id_short and not self._spdx_licenses.get(lic_id_short):
                            self._spdx_licenses[lic_id_short] = lic_id
                    if lic_name:
                        self._spdx_lic_names[lic_name] = lic_id
        return True

    def get_spdx_license_id(self, lic_name: str) -> str:
        """
        Get the SPDX License ID if possible
        :param lic_name: license name or id
        :return: SPDX license identifier or None
        """
        if not lic_name:
            return None
        search_name_no_spaces = re.sub('\\s+', '', lic_name).lower()  # Remove spaces and lowercase the name
        search_name_dashes = re.sub('\\s+', '-', lic_name).lower()  # Replace spaces with dashes and lowercase
        lic_id = self._spdx_licenses.get(search_name_no_spaces)  # Lookup based on license id
        if lic_id:
            return lic_id
        lic_id = self._spdx_licenses.get(search_name_dashes)
        if lic_id:
            return lic_id
        lic_id = self._spdx_lic_names.get(search_name_no_spaces)  # Lookup based on license name
        if lic_id:
            return lic_id
        lic_id = self._spdx_lic_names.get(search_name_dashes)
        if lic_id:
            return lic_id
        self.print_debug(f'Warning: Failed to find valid SPDX license identifier for: {lic_name}')
        return None


#
# End of SpdxLite Class
#



"""
SPDX-License-Identifier: MIT

  Copyright (c) 2024, SCANOSS

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
"""

import json
from typing import Any, Dict, List

from scanoss.utils.abstract_presenter import AbstractPresenter

from .scanossbase import ScanossBase

MATCH_TYPES = ['file', 'snippet']
STATUSES = ['pending', 'identified']


AVAILABLE_FILTER_VALUES = {
    'match_type': [e for e in MATCH_TYPES],
    'status': [e for e in STATUSES],
}


ARG_TO_FILTER_MAP = {
    'match_type': 'id',
    'status': 'status',
}

PENDING_IDENTIFICATION_FILTERS = {
    'match_type': ['file', 'snippet'],
    'status': ['pending'],
}


class ResultsPresenter(AbstractPresenter):
    """
    SCANOSS Results presenter class
    Handles the presentation of the scan results
    """

    def __init__(self, results_instance, **kwargs):
        super().__init__(**kwargs)
        self.results = results_instance

    def _format_json_output(self) -> str:
        """
        Format the output data into a JSON object
        """

        formatted_data = []
        for item in self.results.data:
            formatted_data.append(
                {
                    'file': item.get('filename'),
                    'status': item.get('status', 'N/A'),
                    'match_type': item['id'],
                    'matched': item.get('matched', 'N/A'),
                    'purl': (item.get('purl')[0] if item.get('purl') else 'N/A'),
                    'license': (item.get('licenses')[0].get('name', 'N/A') if item.get('licenses') else 'N/A'),
                }
            )
        try:
            return json.dumps({'results': formatted_data, 'total': len(formatted_data)}, indent=2)
        except Exception as e:
            self.base.print_stderr(f'ERROR: Problem formatting JSON output: {e}')
            return ''

    def _format_cyclonedx_output(self) -> str:
        raise NotImplementedError('CycloneDX output is not implemented')

    def _format_spdxlite_output(self) -> str:
        raise NotImplementedError('SPDXlite output is not implemented')

    def _format_csv_output(self) -> str:
        raise NotImplementedError('CSV output is not implemented')

    def _format_raw_output(self) -> str:
        raise NotImplementedError('Raw output is not implemented')

    def _format_plain_output(self) -> str:
        """Format the output data into a plain text string

        Returns:
            str: The formatted output data
        """
        if not self.results.data:
            msg = 'No results to present'
            return msg

        formatted = ''
        for item in self.results.data:
            formatted += f'{self._format_plain_output_item(item)}\n'
        return formatted

    @staticmethod
    def _format_plain_output_item(item):
        purls = item.get('purl', [])
        licenses = item.get('licenses', [])

        return (
            f'File: {item.get("filename")}\n'
            f'Match type: {item.get("id")}\n'
            f'Status: {item.get("status", "N/A")}\n'
            f'Matched: {item.get("matched", "N/A")}\n'
            f'Purl: {purls[0] if purls else "N/A"}\n'
            f'License: {licenses[0].get("name", "N/A") if licenses else "N/A"}\n'
        )


class Results:
    """
    SCANOSS Results class \n
    Handles the parsing and filtering of the scan results
    """

    def __init__(  # noqa: PLR0913
        self,
        debug: bool = False,
        trace: bool = False,
        quiet: bool = False,
        filepath: str = None,
        match_type: str = None,
        status: str = None,
        output_file: str = None,
        output_format: str = None,
    ):
        """Initialise the Results class

        Args:
            debug (bool, optional): Debug. Defaults to False.
            trace (bool, optional): Trace. Defaults to False.
            quiet (bool, optional): Quiet. Defaults to False.
            filepath (str, optional): Path to the scan results file. Defaults to None.
            match_type (str, optional): Comma separated match type filters. Defaults to None.
            status (str, optional): Comma separated status filters. Defaults to None.
            output_file (str, optional): Path to the output file. Defaults to None.
            output_format (str, optional): Output format. Defaults to None.
        """

        self.base = ScanossBase(debug, trace, quiet)
        self.data = self._load_and_transform(filepath)
        self.filters = self._load_filters(match_type=match_type, status=status)
        self.presenter = ResultsPresenter(
            self,
            debug=debug,
            trace=trace,
            quiet=quiet,
            output_file=output_file,
            output_format=output_format,
        )

    def load_file(self, file: str) -> Dict[str, Any]:
        """Load the JSON file

        Args:
            file (str): Path to the JSON file

        Returns:
            Dict[str, Any]: The parsed JSON data
        """
        with open(file, 'r') as jsonfile:
            try:
                return json.load(jsonfile)
            except Exception as e:
                self.base.print_stderr(f'ERROR: Problem parsing input JSON: {e}')

    def _load_and_transform(self, file: str) -> List[Dict[str, Any]]:
        """
        Load the file and transform the data into a list of dictionaries with the filename and the file data
        """

        raw_data = self.load_file(file)
        return self._transform_data(raw_data)

    @staticmethod
    def _transform_data(data: dict) -> list:
        """Transform the data into a list of dictionaries with the filename and the file data

        Args:
            data (dict): The raw data

        Returns:
            list: The transformed data
        """
        result = []
        for filename, file_data in data.items():
            if file_data:
                file_obj = {'filename': filename}
                file_obj.update(file_data[0])
                result.append(file_obj)
        return result

    def _load_filters(self, **kwargs):
        """Extract and parse the filters

        Returns:
            dict: Parsed filters
        """
        filters = {}

        for key, value in kwargs.items():
            if value:
                filters[key] = self._extract_comma_separated_values(value)

        return filters

    @staticmethod
    def _extract_comma_separated_values(values: str):
        return [value.strip() for value in values.split(',')]

    def apply_filters(self):
        """Apply the filters to the data"""
        filtered_data = []
        for item in self.data:
            if self._item_matches_filters(item):
                filtered_data.append(item)
        self.data = filtered_data

        return self

    def _item_matches_filters(self, item):
        for filter_key, filter_values in self.filters.items():
            if not filter_values:
                continue

            self._validate_filter_values(filter_key, filter_values)

            item_value = item.get(ARG_TO_FILTER_MAP[filter_key])
            if isinstance(filter_values, list):
                if item_value not in filter_values:
                    return False
            elif item_value != filter_values:
                return False
        return True

    @staticmethod
    def _validate_filter_values(filter_key: str, filter_value: List[str]):
        if any(value not in AVAILABLE_FILTER_VALUES.get(filter_key, []) for value in filter_value):
            valid_values = ', '.join(AVAILABLE_FILTER_VALUES.get(filter_key, []))
            raise ValueError(
                f"ERROR: Invalid filter value '{filter_value}' for filter '{filter_key}'. "
                f'Valid values are: {valid_values}'
            )

    def get_pending_identifications(self):
        """Get files with 'pending' status and 'file' or 'snippet' match type"""
        self.filters = PENDING_IDENTIFICATION_FILTERS
        self.apply_filters()

        return self

    def has_results(self):
        return bool(self.data)

    def present(self, output_format: str = None, output_file: str = None):
        """Present the results in the selected format"""
        self.presenter.present(output_format=output_format, output_file=output_file)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: scanoss/api/components/v2/scanoss-components.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'scanoss/api/components/v2/scanoss-components.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from scanoss.api.common.v2 import scanoss_common_pb2 as scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2scanoss/api/components/v2/scanoss-components.proto\x12\x19scanoss.api.components.v2\x1a*scanoss/api/common/v2/scanoss-common.proto\x1a\x1cgoogle/api/annotations.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\"v\n\x11\x43ompSearchRequest\x12\x0e\n\x06search\x18\x01 \x01(\t\x12\x0e\n\x06vendor\x18\x02 \x01(\t\x12\x11\n\tcomponent\x18\x03 \x01(\t\x12\x0f\n\x07package\x18\x04 \x01(\t\x12\r\n\x05limit\x18\x06 \x01(\x05\x12\x0e\n\x06offset\x18\x07 \x01(\x05\"\xca\x01\n\rCompStatistic\x12\x1a\n\x12total_source_files\x18\x01 \x01(\x05\x12\x13\n\x0btotal_lines\x18\x02 \x01(\x05\x12\x19\n\x11total_blank_lines\x18\x03 \x01(\x05\x12\x44\n\tlanguages\x18\x04 \x03(\x0b\x32\x31.scanoss.api.components.v2.CompStatistic.Language\x1a\'\n\x08Language\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05\x66iles\x18\x02 \x01(\x05\"\xfb\x01\n\x15\x43ompStatisticResponse\x12\x45\n\x05purls\x18\x01 \x03(\x0b\x32\x36.scanoss.api.components.v2.CompStatisticResponse.Purls\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1a\x64\n\x05Purls\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12<\n\nstatistics\x18\x03 \x01(\x0b\x32(.scanoss.api.components.v2.CompStatistic\"\xd3\x01\n\x12\x43ompSearchResponse\x12K\n\ncomponents\x18\x01 \x03(\x0b\x32\x37.scanoss.api.components.v2.CompSearchResponse.Component\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1a\x39\n\tComponent\x12\x11\n\tcomponent\x18\x01 \x01(\t\x12\x0c\n\x04purl\x18\x02 \x01(\t\x12\x0b\n\x03url\x18\x03 \x01(\t\"1\n\x12\x43ompVersionRequest\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\r\n\x05limit\x18\x02 \x01(\x05\"\xe4\x03\n\x13\x43ompVersionResponse\x12K\n\tcomponent\x18\x01 \x01(\x0b\x32\x38.scanoss.api.components.v2.CompVersionResponse.Component\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1aO\n\x07License\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07spdx_id\x18\x02 \x01(\t\x12\x18\n\x10is_spdx_approved\x18\x03 \x01(\x08\x12\x0b\n\x03url\x18\x04 \x01(\t\x1ar\n\x07Version\x12\x0f\n\x07version\x18\x01 \x01(\t\x12H\n\x08licenses\x18\x04 \x03(\x0b\x32\x36.scanoss.api.components.v2.CompVersionResponse.License\x12\x0c\n\x04\x64\x61te\x18\x05 \x01(\t\x1a\x83\x01\n\tComponent\x12\x11\n\tcomponent\x18\x01 \x01(\t\x12\x0c\n\x04purl\x18\x02 \x01(\t\x12\x0b\n\x03url\x18\x03 \x01(\t\x12H\n\x08versions\x18\x04 \x03(\x0b\x32\x36.scanoss.api.components.v2.CompVersionResponse.Version2\xcb\x04\n\nComponents\x12p\n\x04\x45\x63ho\x12\".scanoss.api.common.v2.EchoRequest\x1a#.scanoss.api.common.v2.EchoResponse\"\x1f\x82\xd3\xe4\x93\x02\x19\x12\x17/api/v2/components/echo\x12\x92\x01\n\x10SearchComponents\x12,.scanoss.api.components.v2.CompSearchRequest\x1a-.scanoss.api.components.v2.CompSearchResponse\"!\x82\xd3\xe4\x93\x02\x1b\x12\x19/api/v2/components/search\x12\x9a\x01\n\x14GetComponentVersions\x12-.scanoss.api.components.v2.CompVersionRequest\x1a..scanoss.api.components.v2.CompVersionResponse\"#\x82\xd3\xe4\x93\x02\x1d\x12\x1b/api/v2/components/versions\x12\x98\x01\n\x16GetComponentStatistics\x12\".scanoss.api.common.v2.PurlRequest\x1a\x30.scanoss.api.components.v2.CompStatisticResponse\"(\x82\xd3\xe4\x93\x02\"\"\x1d/api/v2/components/statistics:\x01*B\x9a\x03Z5github.com/scanoss/papi/api/componentsv2;componentsv2\x92\x41\xdf\x02\x12\x9d\x01\n\x1aSCANOSS Components Service\x12(Provides component intelligence services\"P\n\x12scanoss-components\x12%https://github.com/scanoss/components\x1a\x13support@scanoss.com2\x03\x32.0\x1a\x0f\x61pi.scanoss.com*\x02\x02\x01\x32\x10\x61pplication/json:\x10\x61pplication/jsonR;\n\x03\x34\x30\x34\x12\x34\n*Returned when the resource does not exist.\x12\x06\n\x04\x9a\x02\x01\x07Z8\n6\n\x07\x61pi_key\x12+\x08\x02\x12\x1a\x41PI key for authentication\x1a\tx-api-key \x02\x62\r\n\x0b\n\x07\x61pi_key\x12\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scanoss.api.components.v2.scanoss_components_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z5github.com/scanoss/papi/api/componentsv2;componentsv2\222A\337\002\022\235\001\n\032SCANOSS Components Service\022(Provides component intelligence services\"P\n\022scanoss-components\022%https://github.com/scanoss/components\032\023support@scanoss.com2\0032.0\032\017api.scanoss.com*\002\002\0012\020application/json:\020application/jsonR;\n\003404\0224\n*Returned when the resource does not exist.\022\006\n\004\232\002\001\007Z8\n6\n\007api_key\022+\010\002\022\032API key for authentication\032\tx-api-key \002b\r\n\013\n\007api_key\022\000'
  _globals['_COMPONENTS'].methods_by_name['Echo']._loaded_options = None
  _globals['_COMPONENTS'].methods_by_name['Echo']._serialized_options = b'\202\323\344\223\002\031\022\027/api/v2/components/echo'
  _globals['_COMPONENTS'].methods_by_name['SearchComponents']._loaded_options = None
  _globals['_COMPONENTS'].methods_by_name['SearchComponents']._serialized_options = b'\202\323\344\223\002\033\022\031/api/v2/components/search'
  _globals['_COMPONENTS'].methods_by_name['GetComponentVersions']._loaded_options = None
  _globals['_COMPONENTS'].methods_by_name['GetComponentVersions']._serialized_options = b'\202\323\344\223\002\035\022\033/api/v2/components/versions'
  _globals['_COMPONENTS'].methods_by_name['GetComponentStatistics']._loaded_options = None
  _globals['_COMPONENTS'].methods_by_name['GetComponentStatistics']._serialized_options = b'\202\323\344\223\002\"\"\035/api/v2/components/statistics:\001*'
  _globals['_COMPSEARCHREQUEST']._serialized_start=203
  _globals['_COMPSEARCHREQUEST']._serialized_end=321
  _globals['_COMPSTATISTIC']._serialized_start=324
  _globals['_COMPSTATISTIC']._serialized_end=526
  _globals['_COMPSTATISTIC_LANGUAGE']._serialized_start=487
  _globals['_COMPSTATISTIC_LANGUAGE']._serialized_end=526
  _globals['_COMPSTATISTICRESPONSE']._serialized_start=529
  _globals['_COMPSTATISTICRESPONSE']._serialized_end=780
  _globals['_COMPSTATISTICRESPONSE_PURLS']._serialized_start=680
  _globals['_COMPSTATISTICRESPONSE_PURLS']._serialized_end=780
  _globals['_COMPSEARCHRESPONSE']._serialized_start=783
  _globals['_COMPSEARCHRESPONSE']._serialized_end=994
  _globals['_COMPSEARCHRESPONSE_COMPONENT']._serialized_start=937
  _globals['_COMPSEARCHRESPONSE_COMPONENT']._serialized_end=994
  _globals['_COMPVERSIONREQUEST']._serialized_start=996
  _globals['_COMPVERSIONREQUEST']._serialized_end=1045
  _globals['_COMPVERSIONRESPONSE']._serialized_start=1048
  _globals['_COMPVERSIONRESPONSE']._serialized_end=1532
  _globals['_COMPVERSIONRESPONSE_LICENSE']._serialized_start=1203
  _globals['_COMPVERSIONRESPONSE_LICENSE']._serialized_end=1282
  _globals['_COMPVERSIONRESPONSE_VERSION']._serialized_start=1284
  _globals['_COMPVERSIONRESPONSE_VERSION']._serialized_end=1398
  _globals['_COMPVERSIONRESPONSE_COMPONENT']._serialized_start=1401
  _globals['_COMPVERSIONRESPONSE_COMPONENT']._serialized_end=1532
  _globals['_COMPONENTS']._serialized_start=1535
  _globals['_COMPONENTS']._serialized_end=2122
# @@protoc_insertion_point(module_scope)



"""
SPDX-License-Identifier: MIT

  Copyright (c) 2025, SCANOSS

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
"""

import json
from dataclasses import dataclass
from typing import Any, Dict, List

from ..policy_check import PolicyStatus
from .raw_base import RawBase


@dataclass
class License:
    spdxid: str
    copyleft: bool
    url: str
    source: str

@dataclass
class Component:
    purl: str
    version: str
    licenses: List[License]
    status: str

class Copyleft(RawBase[Component]):
    """
    SCANOSS Copyleft class
    Inspects components for copyleft licenses
    """

    def __init__( # noqa: PLR0913
        self,
        debug: bool = False,
        trace: bool = False,
        quiet: bool = False,
        filepath: str = None,
        format_type: str = 'json',
        status: str = None,
        output: str = None,
        include: str = None,
        exclude: str = None,
        explicit: str = None,
    ):
        """
        Initialise the Copyleft class.

        :param debug: Enable debug mode
        :param trace: Enable trace mode (default True)
        :param quiet: Enable quiet mode
        :param filepath: Path to the file containing component data
        :param format_type: Output format ('json' or 'md')
        :param status: Path to save the status output
        :param output: Path to save detailed output
        :param include: Licenses to include in the analysis
        :param exclude: Licenses to exclude from the analysis
        :param explicit: Explicitly defined licenses
        """
        super().__init__(debug, trace, quiet, format_type,filepath, output ,status, name='Copyleft Policy')
        self.license_util.init(include, exclude, explicit)
        self.filepath = filepath
        self.format = format
        self.output = output
        self.status = status
        self.include = include
        self.exclude = exclude
        self.explicit = explicit

    def _json(self, components: list[Component]) -> Dict[str, Any]:
        """
        Format the components with copyleft licenses as JSON.

        :param components: List of components with copyleft licenses
        :return: Dictionary with formatted JSON details and summary
        """
        # A component is considered unique by its combination of PURL (Package URL) and license
        component_licenses = self._group_components_by_license(components)
        details = {}
        if len(components) > 0:
            details = {'components': components}
        return {
            'details': f'{json.dumps(details, indent=2)}\n',
            'summary': f'{len(component_licenses)} component(s) with copyleft licenses were found.\n',
        }

    def _markdown(self, components: list[Component]) -> Dict[str, Any]:
        """
        Format the components with copyleft licenses as Markdown.

        :param components: List of components with copyleft licenses
        :return: Dictionary with formatted Markdown details and summary
        """
        return self._md_summary_generator(components, self.generate_table)

    def _jira_markdown(self, components: list[Component]) -> Dict[str, Any]:
        """
        Format the components with copyleft licenses as Markdown.

        :param components: List of components with copyleft licenses
        :return: Dictionary with formatted Markdown details and summary
        """
        return self._md_summary_generator(components, self.generate_jira_table)

    def _md_summary_generator(self, components: list[Component], table_generator):
        """
        Generates a Markdown summary for components with a focus on copyleft licenses.

        This function processes a list of components and groups them by their licenses.
        For each group, the components are mapped with their license data and a tabular representation is created.
        The generated Markdown summary includes a detailed table and a summary overview.

        Parameters:
        components: list[Component]
            A list of Component objects to process for generating the summary.
        table_generator
            A callable function to generate tabular data for components.

        Returns:
        dict
            A dictionary containing two keys:
            - 'details': A detailed Markdown representation including a table of components
              and associated copyleft license data.
            - 'summary': A textual summary highlighting the total number of components
              with copyleft licenses.
        """
        # A component is considered unique by its combination of PURL (Package URL) and license
        component_licenses = self._group_components_by_license(components)
        headers = ['Component', 'License', 'URL', 'Copyleft']
        centered_columns = [1, 4]
        rows = []
        for comp_lic_item in component_licenses:
            row = [
                comp_lic_item['purl'],
                comp_lic_item['spdxid'],
                comp_lic_item['url'],
                'YES' if comp_lic_item['copyleft'] else 'NO',
            ]
            rows.append(row)
        # End license loop
        # End component loop
        return {
            'details': f'### Copyleft Licenses\n{table_generator(headers, rows, centered_columns)}',
            'summary': f'{len(component_licenses)} component(s) with copyleft licenses were found.\n',
        }

    def _get_components_with_copyleft_licenses(self, components: list) -> list[Dict]:
        """
        Filter the components list to include only those with copyleft licenses.

        :param components: List of all components
        :return: List of components with copyleft licenses
        """
        filtered_components = []
        for component in components:
            copyleft_licenses = [lic for lic in component['licenses'] if lic['copyleft']]
            if copyleft_licenses:
                # Remove unused keys
                del component['count']
                del component['declared']
                del component['undeclared']
                filtered_component = component
                # Remove 'count' from each license using pop
                for lic in copyleft_licenses:
                    lic.pop('count', None)  # None is default value if key doesn't exist

                filtered_component['licenses'] = copyleft_licenses
                filtered_components.append(filtered_component)
        # End component loop
        self.print_debug(f'Copyleft components: {filtered_components}')
        return filtered_components

    def _get_components(self):
        """
        Extract and process components from results and their dependencies.

        This method performs the following steps:
        1. Validates that `self.results` is loaded. Returns `None` if not.
        2. Extracts file, snippet, and dependency components into a dictionary.
        3. Converts components to a list and processes their licenses.

        :return: A list of processed components with license data, or `None` if `self.results` is not set.
        """
        if self.results is None:
            return None

        components: dict = {}
        # Extract component and license data from file and dependency results. Both helpers mutate `components`
        self._get_components_data(self.results, components)
        self._get_dependencies_data(self.results, components)
        return self._convert_components_to_list(components)

    def run(self):
        """
        Run the copyleft license inspection process.

        This method performs the following steps:
        1. Get all components
        2. Filter components with copyleft licenses
        3. Format the results
        4. Save the output to files if required

        :return: Dictionary containing the inspection results
        """
        self._debug()
        # Get the components from the results
        components = self._get_components()
        if components is None:
            return PolicyStatus.ERROR.value, {}
        # Get a list of copyleft components if they exist
        copyleft_components = self._get_components_with_copyleft_licenses(components)
        # Format the results and save to files if required
        return self._generate_formatter_report(copyleft_components)
#
# End of Copyleft Class
#



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: scanoss/api/semgrep/v2/scanoss-semgrep.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'scanoss/api/semgrep/v2/scanoss-semgrep.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from scanoss.api.common.v2 import scanoss_common_pb2 as scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,scanoss/api/semgrep/v2/scanoss-semgrep.proto\x12\x16scanoss.api.semgrep.v2\x1a*scanoss/api/common/v2/scanoss-common.proto\x1a\x1cgoogle/api/annotations.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\"\x96\x03\n\x0fSemgrepResponse\x12<\n\x05purls\x18\x01 \x03(\x0b\x32-.scanoss.api.semgrep.v2.SemgrepResponse.Purls\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1a\x43\n\x05Issue\x12\x0e\n\x06ruleID\x18\x01 \x01(\t\x12\x0c\n\x04\x66rom\x18\x02 \x01(\t\x12\n\n\x02to\x18\x03 \x01(\t\x12\x10\n\x08severity\x18\x04 \x01(\t\x1a\x64\n\x04\x46ile\x12\x0f\n\x07\x66ileMD5\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\t\x12=\n\x06issues\x18\x03 \x03(\x0b\x32-.scanoss.api.semgrep.v2.SemgrepResponse.Issue\x1a\x63\n\x05Purls\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12;\n\x05\x66iles\x18\x03 \x03(\x0b\x32,.scanoss.api.semgrep.v2.SemgrepResponse.File2\xf8\x01\n\x07Semgrep\x12p\n\x04\x45\x63ho\x12\".scanoss.api.common.v2.EchoRequest\x1a#.scanoss.api.common.v2.EchoResponse\"\x1f\x82\xd3\xe4\x93\x02\x19\"\x14/api/v2/semgrep/echo:\x01*\x12{\n\tGetIssues\x12\".scanoss.api.common.v2.PurlRequest\x1a\'.scanoss.api.semgrep.v2.SemgrepResponse\"!\x82\xd3\xe4\x93\x02\x1b\"\x16/api/v2/semgrep/issues:\x01*B\x85\x02Z/github.com/scanoss/papi/api/semgrepv2;semgrepv2\x92\x41\xd0\x01\x12j\n\x17SCANOSS Semgrep Service\"J\n\x0fscanoss-semgrep\x12\"https://github.com/scanoss/semgrep\x1a\x13support@scanoss.com2\x03\x32.0*\x01\x01\x32\x10\x61pplication/json:\x10\x61pplication/jsonR;\n\x03\x34\x30\x34\x12\x34\n*Returned when the resource does not exist.\x12\x06\n\x04\x9a\x02\x01\x07\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scanoss.api.semgrep.v2.scanoss_semgrep_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z/github.com/scanoss/papi/api/semgrepv2;semgrepv2\222A\320\001\022j\n\027SCANOSS Semgrep Service\"J\n\017scanoss-semgrep\022\"https://github.com/scanoss/semgrep\032\023support@scanoss.com2\0032.0*\001\0012\020application/json:\020application/jsonR;\n\003404\0224\n*Returned when the resource does not exist.\022\006\n\004\232\002\001\007'
  _globals['_SEMGREP'].methods_by_name['Echo']._loaded_options = None
  _globals['_SEMGREP'].methods_by_name['Echo']._serialized_options = b'\202\323\344\223\002\031\"\024/api/v2/semgrep/echo:\001*'
  _globals['_SEMGREP'].methods_by_name['GetIssues']._loaded_options = None
  _globals['_SEMGREP'].methods_by_name['GetIssues']._serialized_options = b'\202\323\344\223\002\033\"\026/api/v2/semgrep/issues:\001*'
  _globals['_SEMGREPRESPONSE']._serialized_start=195
  _globals['_SEMGREPRESPONSE']._serialized_end=601
  _globals['_SEMGREPRESPONSE_ISSUE']._serialized_start=331
  _globals['_SEMGREPRESPONSE_ISSUE']._serialized_end=398
  _globals['_SEMGREPRESPONSE_FILE']._serialized_start=400
  _globals['_SEMGREPRESPONSE_FILE']._serialized_end=500
  _globals['_SEMGREPRESPONSE_PURLS']._serialized_start=502
  _globals['_SEMGREPRESPONSE_PURLS']._serialized_end=601
  _globals['_SEMGREP']._serialized_start=604
  _globals['_SEMGREP']._serialized_end=852
# @@protoc_insertion_point(module_scope)



"""
SPDX-License-Identifier: MIT

  Copyright (c) 2024, SCANOSS

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
"""

from typing import List, Tuple

from packageurl import PackageURL
from packageurl.contrib import purl2url

from .scanoss_settings import BomEntry, ScanossSettings
from .scanossbase import ScanossBase


def _get_match_type_message(result_path: str, bom_entry: BomEntry, action: str) -> str:
    """
    Compose message based on match type

    Args:
        result_path (str): Path of the scan result
        bom_entry (BomEntry): BOM entry to compare with
        action (str): Post processing action being performed

    Returns:
        str: The message to be printed
    """
    if bom_entry.get('path') and bom_entry.get('purl'):
        message = f"{action} '{result_path}'. Full match found."
    elif bom_entry.get('purl'):
        message = f"{action} '{result_path}'. Found PURL match."
    else:
        message = f"{action} '{result_path}'. Found path match."
    return message


def _is_full_match(result_path: str, result_purls: List[str], bom_entry: BomEntry) -> bool:
    """
    Check if path and purl matches fully with the bom entry

    Args:
        result_path (str): Scan result path
        result_purls (List[str]): Scan result purls
        bom_entry (BomEntry): BOM entry to compare with

    Returns:
        bool: True if the path and purl match, False otherwise
    """
    if not result_purls:
        return False
    return bool(
        (bom_entry.get('purl') and bom_entry.get('path'))
        and (bom_entry.get('path') == result_path)
        and (bom_entry.get('purl') in result_purls)
    )


class ScanPostProcessor(ScanossBase):
    """
    Handles post-processing of the scan results
    """

    def __init__(
        self,
        scan_settings: ScanossSettings,
        debug: bool = False,
        trace: bool = False,
        quiet: bool = False,
        results: dict = None,
    ):
        """
        Args:
            scan_settings (ScanossSettings): Scan settings object
            debug (bool, optional): Debug mode. Defaults to False.
            trace (bool, optional): Traces. Defaults to False.
            quiet (bool, optional): Quiet mode. Defaults to False.
            results (dict | str, optional): Results to be processed. Defaults to None.
        """
        super().__init__(debug, trace, quiet)
        self.scan_settings = scan_settings
        self.results: dict = results
        self.component_info_map: dict = {}

    def load_results(self, raw_results: dict):
        """Load the raw results

        Args:
            raw_results (dict): Raw scan results
        """
        self.results = raw_results
        self._load_component_info()
        return self

    def _load_component_info(self):
        """Create a map of component information from scan results for faster lookup"""
        if not self.results:
            return
        for _, result in self.results.items():
            result = result[0] if isinstance(result, list) else result
            purls = result.get('purl', [])
            for purl in purls:
                self.component_info_map[purl] = result

    def post_process(self):
        """
        Post-process the scan results

        Returns:
            dict: Processed results
        """
        if self.scan_settings.is_legacy():
            self.print_stderr(
                'Legacy settings file detected. Post-processing is not supported for legacy settings file.'
            )
            return self.results
        self._remove_dismissed_files()
        self._replace_purls()
        return self.results

    def _remove_dismissed_files(self):
        """
        Remove entries from the results based on files and/or purls specified in the SCANOSS settings file
        """
        to_remove_entries = self.scan_settings.get_bom_remove()
        if not to_remove_entries:
            return
        self.results = {
            result_path: result
            for result_path, result in self.results.items()
            if not self._should_remove_result(result_path, result, to_remove_entries)
        }

    def _replace_purls(self):
        """
        Replace purls in the results based on the SCANOSS settings file
        """
        to_replace_entries = self.scan_settings.get_bom_replace()
        if not to_replace_entries:
            return

        for result_path, result in self.results.items():
            result = result[0] if isinstance(result, list) else result
            should_replace, to_replace_with_purl = self._should_replace_result(result_path, result, to_replace_entries)
            if should_replace:
                self.results[result_path] = [self._update_replaced_result(result, to_replace_with_purl)]

    def _update_replaced_result(self, result: dict, to_replace_with_purl: str) -> dict:
        """
        Update the result with the new purl and component information if available,
        otherwise removes the old component information

        Args:
            result (dict): The result to update
            to_replace_with_purl (str): The purl to replace with

        Returns:
            dict: Updated result
        """

        if self.component_info_map.get(to_replace_with_purl):
            result.update(self.component_info_map[to_replace_with_purl])
        else:
            try:
                new_component = PackageURL.from_string(to_replace_with_purl).to_dict()
                new_component_url = purl2url.get_repo_url(to_replace_with_purl)
            except RuntimeError:
                self.print_stderr(
                    f"ERROR: Issue while replacing: Invalid PURL '{to_replace_with_purl}' in settings file. Skipping."
                )
                return result

            result['component'] = new_component.get('name')
            result['url'] = new_component_url
            result['vendor'] = new_component.get('namespace')

            result.pop('licenses', None)
            result.pop('file', None)
            result.pop('file_hash', None)
            result.pop('file_url', None)
            result.pop('latest', None)
            result.pop('release_date', None)
            result.pop('source_hash', None)
            result.pop('url_hash', None)
            result.pop('url_stats', None)
            result.pop('url_stats', None)
            result.pop('version', None)

        result['purl'] = [to_replace_with_purl]

        return result

    def _should_replace_result(
        self, result_path: str, result: dict, to_replace_entries: List[BomEntry]
    ) -> Tuple[bool, str]:
        """
        Check if a result should be replaced based on the SCANOSS settings

        Args:
            result_path (str): Path of the result data
            result (dict): Result to check
            to_replace_entries (List[BomEntry]): BOM entries to replace from the settings file

        Returns:
            bool: True if the result should be replaced, False otherwise
            str: The purl to replace with
        """
        result_purls = result.get('purl', [])
        for to_replace_entry in to_replace_entries:
            to_replace_path = to_replace_entry.get('path')
            to_replace_purl = to_replace_entry.get('purl')
            to_replace_with = to_replace_entry.get('replace_with')

            if not to_replace_path and not to_replace_purl or not to_replace_with:
                continue
            if (
                _is_full_match(result_path, result_purls, to_replace_entry)
                or (not to_replace_path and to_replace_purl in result_purls)
                or (not to_replace_purl and to_replace_path == result_path)
            ):
                self._print_message(result_path, result_purls, to_replace_entry, 'Replacing')
                return True, to_replace_with

        return False, None

    def _should_remove_result(self, result_path: str, result: dict, to_remove_entries: List[BomEntry]) -> bool:
        """
        Check if a result should be removed based on the SCANOSS settings

        :param result_path: path of the result data
        :param result: result to check
        :param to_remove_entries: BOM entries to remove from the result
        :return:
        """
        result = result[0] if isinstance(result, list) else result
        result_purls = result.get('purl', [])

        for to_remove_entry in to_remove_entries:
            to_remove_path = to_remove_entry.get('path')
            to_remove_purl = to_remove_entry.get('purl')

            if not to_remove_path and not to_remove_purl:
                continue
            if (
                _is_full_match(result_path, result_purls, to_remove_entry)
                or (not to_remove_path and to_remove_purl in result_purls)
                or (not to_remove_purl and to_remove_path == result_path)
            ):
                self._print_message(result_path, result_purls, to_remove_entry, 'Removing')
                return True

        return False

    def _print_message(self, result_path: str, result_purls: List[str], bom_entry: BomEntry, action: str) -> None:
        """
        Print a message about replacing or removing a result

        :param result_path:
        :param result_purls:
        :param bom_entry:
        :param action:
        :return:
        """
        message = (
            f'{_get_match_type_message(result_path, bom_entry, action)} \n'
            f'Details:\n'
            f'  - PURLs: {", ".join(result_purls)}\n'
            f"  - Path: '{result_path}'\n"
        )
        if action == 'Replacing':
            message += f" - {action} with '{bom_entry.get('replace_with')}'"
        self.print_debug(message)


#
# End of ScanPostProcessor Class
#



"""
SPDX-License-Identifier: MIT

  Copyright (c) 2025, SCANOSS

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
"""

import re
import unicodedata

FNV64_OFFSET_BASIS = 14695981039346656037
FNV64_PRIME = 1099511628211
MASK64 = 0xFFFFFFFFFFFFFFFF


def fnv1_64(data: bytes) -> int:
    """Compute the 64‐bit FNV‑1 hash of data."""
    h = FNV64_OFFSET_BASIS
    for b in data:
        h = (h * FNV64_PRIME) & MASK64
        h = h ^ b
    return h


class SimhashFeature:
    def __init__(self, hash_value: int, weight: int = 1):
        self.hash_value = hash_value
        self.weight = weight

    def sum(self) -> int:
        """Return the 64-bit hash (sum) of this feature."""
        return self.hash_value

    def get_weight(self) -> int:
        """Return the weight of this feature."""
        return self.weight


def new_feature(f: bytes) -> SimhashFeature:
    """Return a new feature for the given byte slice with weight 1."""
    return SimhashFeature(fnv1_64(f), 1)


def new_feature_with_weight(f: bytes, weight: int) -> SimhashFeature:
    """Return a new feature for the given byte slice with the given weight."""
    return SimhashFeature(fnv1_64(f), weight)


def vectorize(features: list) -> list:
    """
    Given a list of features, return a 64-element vector.
    Each feature contributes its weight to each coordinate,
    added if that bit is set and subtracted otherwise.
    """
    v = [0] * 64
    for feature in features:
        h = feature.sum()
        w = feature.get_weight()
        for i in range(64):
            if ((h >> i) & 1) == 1:
                v[i] += w
            else:
                v[i] -= w
    return v


def vectorize_bytes(features: list) -> list:
    """
    Given a list of byte slices, treat each as a feature (with weight 1)
    by computing its FNV-1 hash.
    """
    v = [0] * 64
    for feat in features:
        h = fnv1_64(feat)
        for i in range(64):
            if ((h >> i) & 1) == 1:
                v[i] += 1
            else:
                v[i] -= 1
    return v


def fingerprint(v: list) -> int:
    """
    Given a 64-element vector, return a 64-bit fingerprint.
    For each bit i, if v[i] >= 0, set bit i to 1; otherwise leave it 0.
    """
    f = 0
    for i in range(64):
        if v[i] >= 0:
            f |= 1 << i
    return f


def compare(a: int, b: int) -> int:
    """
    Calculate the Hamming distance between two 64-bit integers.
    (The number of differing bits.)
    """
    v = a ^ b
    c = 0
    while v:
        v &= v - 1
        c += 1
    return c


def simhash(fs) -> int:
    """
    Given a feature set (an object with a get_features() method),
    return its 64-bit simhash.
    """
    return fingerprint(vectorize(fs.get_features()))


def simhash_bytes(b: list) -> int:
    """
    Given a list of byte slices, return the simhash.
    """
    return fingerprint(vectorize_bytes(b))


boundaries = re.compile(rb"[\w']+(?:\://[\w\./]+){0,1}")
unicode_boundaries = re.compile(r"[\w'-]+", re.UNICODE)


# --- Helper Functions for Feature Extraction ---
def _get_features_bytes(b: bytes, pattern: re.Pattern) -> list:
    """
    Split the given byte string using the given regex pattern,
    and return a list of features (each created with new_feature).
    """
    words = pattern.findall(b)
    return [new_feature(word) for word in words]


def _get_features_str(s: str, pattern) -> list:
    """
    Split the given string using the given regex pattern,
    and return a list of features (each created by encoding to UTF-8).
    """
    words = pattern.findall(s)
    return [new_feature(word.encode('utf-8')) for word in words]


class WordFeatureSet:
    def __init__(self, b: bytes):
        # Normalize the input to lowercase.
        self.b = b.lower()

    def get_features(self) -> list:
        return _get_features_bytes(self.b, boundaries)


class UnicodeWordFeatureSet:
    def __init__(self, b: bytes, norm_form: str = 'NFC'):
        # Decode, normalize (using the provided form), and lowercase.
        text = b.decode('utf-8')
        normalized = unicodedata.normalize(norm_form, text)
        self.text = normalized.lower()

    def get_features(self) -> list:
        return _get_features_str(self.text, unicode_boundaries)


def shingle(w: int, b: list) -> list:
    """
    Return the w-shingling of the given set of byte slices.
    For example, if b is [b"this", b"is", b"a", b"test"]
    and w == 2, the result is [b"this is", b"is a", b"a test"].
    """
    if w < 1:
        raise ValueError('simhash.shingle(): k must be a positive integer')
    if w == 1:
        return b
    w = min(w, len(b))
    count = len(b) - w + 1
    shingles = []
    for i in range(count):
        shingles.append(b' '.join(b[i : i + w]))
    return shingles



"""
SPDX-License-Identifier: MIT

  Copyright (c) 2021, SCANOSS

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
"""

import json
import os.path
import subprocess

from .scanossbase import ScanossBase


class ScancodeDeps(ScanossBase):
    """
    SCANOSS dependency scanning class
    """

    def __init__(
        self,
        debug: bool = False,
        quiet: bool = False,
        trace: bool = False,
        output_file: str = None,
        scan_output: str = None,
        timeout: int = 600,
        sc_command: str = None,
    ):
        """
        Initialise ScancodeDeps class
        """
        super().__init__(debug, trace, quiet)
        self.quiet = quiet
        self.debug = debug
        self.trace = trace
        self.timeout = timeout
        self.scan_output = scan_output
        self.sc_command = sc_command if sc_command else 'scancode'
        self.output_file = output_file if output_file else 'scancode-dependencies.json'

    def __log_result(self, string, outfile=None):
        """
        Logs result to file or STDOUT
        """
        if not outfile and self.scan_output:
            outfile = self.scan_output
        if outfile:
            with open(outfile, 'a') as rf:
                rf.write(string + '\n')
        else:
            print(string)

    def remove_interim_file(self, output_file: str = None):
        """
        Remove the temporary Scancode interim file
        :param output_file: filename to remove (optional)
        """
        if not output_file and self.output_file:
            output_file = self.output_file
        if os.path.isfile(output_file):
            try:
                self.print_trace(f'Cleaning temporary scancode files...')
                os.remove(output_file)
            except Exception as e:
                self.print_stderr(f'Warning: Failed to remove temporary file {output_file}: {e}')

    def produce_from_json(self, data: json) -> dict:
        """
        Parse the given input JSON string and return Dependency summary
        :param data: json - JSON object
        :return: Dependency dictionary
        """
        if not data:
            self.print_stderr('ERROR: No JSON data provided to parse.')
            return None
        self.print_debug(f'Processing Scancode results into Dependency data...')
        files = []
        for t in data:
            if t == 'files':  # Only interested in 'files' details
                files_details = data.get(t)
                if not files_details or files_details == '':
                    continue
                # print(f'File: {t}: {file_details}')
                for fd in files_details:
                    # print(f'FD: {fd}')
                    f_path = fd.get('path')
                    if not f_path or f_path == '':
                        continue
                    f_type = fd.get('type')
                    if not f_type or f_type == '' or f_type != 'file':  # Only process files
                        continue
                    f_packages = fd.get('package_data')  # scancode format 2.0
                    if not f_packages or f_packages == '':
                        f_packages = fd.get('packages')  # scancode formate 1.0
                        if not f_packages or f_packages == '':
                            continue
                    self.print_debug(f'Path: {f_path}, Packages: {len(f_packages)}')
                    purls = []
                    scopes = []
                    for pkgs in f_packages:
                        pk_deps = pkgs.get('dependencies')

                        if not pk_deps or pk_deps == '':
                            continue
                        for d in pk_deps:
                            dp = d.get('purl')
                            if not dp or dp == '':
                                continue

                            dp = dp.replace('"', '').replace('%22', '')  # remove unwanted quotes on purls
                            dp_data = {'purl': dp}
                            rq = d.get('extracted_requirement')  # scancode format 2.0
                            if not rq or rq == '':
                                rq = d.get('requirement')  # scancode format 1.0
                            # skip requirement if it ends with the purl (i.e. exact version) or if it's local (file)
                            if rq and rq != '' and not dp.endswith(rq) and not rq.startswith('file:'):
                                dp_data['requirement'] = rq

                            # Gets dependency scope
                            scope = d.get('scope')
                            if scope and scope != '':
                                dp_data['scope'] = scope

                            purls.append(dp_data)
                        # end for loop

                    if len(purls) > 0:
                        files.append({'file': f_path, 'purls': purls})
                    # End packages
                # End file details
        # End dependencies json
        deps = {'files': files}
        return deps

    def produce_from_file(self, json_file: str = None) -> json:
        """
        Parse input JSON dependencies file and produce SCANOSS dependency JSON output
        :param json_file:
        :return: SCANOSS dependency JSON
        """
        if not json_file and self.output_file:
            json_file = self.output_file
        if not json_file:
            self.print_stderr('ERROR: No JSON file provided to parse.')
            return None
        if not os.path.isfile(json_file):
            self.print_stderr(f'ERROR: JSON file does not exist or is not a file: {json_file}')
            return None
        with open(json_file, 'r') as f:
            return self.produce_from_str(f.read())

    def produce_from_str(self, json_str: str) -> dict:
        """
        Parse input JSON dependencies string and produce SCANOSS dependency JSON output
        :param json_str: input JSON string
        :return: SCANOSS dependency JSON
        """
        if not json_str:
            self.print_stderr('ERROR: No JSON string provided to parse.')
            return None
        try:
            data = json.loads(json_str)
        except Exception as e:
            self.print_stderr(f'ERROR: Problem parsing input JSON: {e}')
            return None
        return self.produce_from_json(data)

    def get_dependencies(self, output_file: str = None, what_to_scan: str = None, result_output: str = None) -> bool:
        """
        Get the dependencies for the required file/directory and output the JSON results
        :param output_file:  temporary scanocde file to write interim results to
        :param what_to_scan: file or directory to scan
        :param result_output: output location for parsed JSON dependencies (default: stdout)
        :return: True on success, False otherwise
        """
        self.print_msg('Searching for dependencies...')
        if not self.run_scan(output_file, what_to_scan):
            return False
        self.print_msg('Producing summary...')
        deps = self.produce_from_file(output_file)
        deps = self.__remove_dep_scope(deps)
        self.remove_interim_file(output_file)
        if not deps:
            return False
        self.__log_result(json.dumps(deps, indent=2, sort_keys=True), outfile=result_output)
        return True

    def run_scan(self, output_file: str = None, what_to_scan: str = None) -> bool:
        """
        Run a scan of the specified file/folder and output the results to temporary file
        :param output_file: temporary scancode output filename
        :param what_to_scan: file/directory to scan
        :return: True on success, False otherwise
        """
        if not output_file and self.output_file:
            output_file = self.output_file
        try:
            open(output_file, 'w').close()
            self.print_trace(
                f'About to execute {self.sc_command} -p --only-findings --quiet --json {output_file} {what_to_scan}'
            )
            result = subprocess.run(
                [
                    self.sc_command,
                    '-p',
                    '--only-findings',
                    '--quiet',
                    '--strip-root',
                    '--json',
                    output_file,
                    what_to_scan,
                ],
                cwd=os.getcwd(),
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                text=True,
                timeout=self.timeout,
            )
            self.print_trace(f'Subprocess return: {result}')
            if result.returncode:
                self.print_stderr(
                    f'ERROR: Scancode dependency scan of {what_to_scan} failed with exit code'
                    f' {result.returncode}:\n{result.stdout}'
                )
                return False
        except subprocess.TimeoutExpired as e:
            self.print_stderr(f'ERROR: Timed out attempting to run scancode dependency scan on {what_to_scan}: {e}')
            return False
        except Exception as e:
            self.print_stderr(f'ERROR: Issue running scancode dependency scan on {what_to_scan}: {e}')
            return False
        return True

    def load_from_file(self, json_file: str = None) -> json:
        """
        Load the parsed JSON dependencies file and return the json object
        :param json_file: dependency json file
        :return: SCANOSS dependency JSON
        """
        if not json_file:
            self.print_stderr('ERROR: No parsed JSON file provided to load.')
            return None
        if not os.path.isfile(json_file):
            self.print_stderr(f'ERROR: parsed JSON file does not exist or is not a file: {json_file}')
            return None
        with open(json_file, 'r') as f:
            try:
                return json.loads(f.read())
            except Exception as e:
                self.print_stderr(f'ERROR: Problem loading input JSON: {e}')
        return None

    @staticmethod
    def __remove_dep_scope(deps: json) -> json:
        """
        :param deps: dependencies with scopes
        :return dependencies without scopes
        """
        files = deps.get('files')
        for file in files:
            if 'purls' in file:
                purls = file.get('purls')
                for purl in purls:
                    purl.pop('scope', None)

        return {'files': files}


#
# End of ScancodeDeps Class
#
